<?php

namespace Drupal\fluent;

use Drupal\Core\Field\FieldItemInterface;
use Drupal\Core\Field\FieldItemListInterface;
use Drupal\Core\TypedData\PrimitiveBase;
use Drupal\fluent\Plugin\FluentFieldItemResolverPluginManager;
use Illuminate\Support\Collection as BaseCollection;

class FieldItemResolver
{

  /**
   * @var \Drupal\fluent\Plugin\FluentFieldItemResolverPluginManager
   */
  private FluentFieldItemResolverPluginManager $pluginManager;

  private array $fieldResolvers = [];

  public function __construct(FluentFieldItemResolverPluginManager $plugin) {
    $this->pluginManager = $plugin;

    foreach ($this->pluginManager->getDefinitions() as $id => $definition) {
      $this->fieldResolvers[$id] = $this->pluginManager->createInstance($id);
    }
  }

  public function resolve($field, array $path, $defaultValue = NULL) {

    if ($field instanceof FieldItemListInterface && $field->isEmpty()) {
      $default = $field->getFieldDefinition()->getDefaultValueLiteral();

      $end = end($path);
      if (!empty($default) && array_key_exists($end, $default[0])) {
        return $default[0][$end];
      }

      return $defaultValue;
    }

    // TODO: Create plugin for each @FieldType
    if ($field instanceof FieldItemInterface || $field instanceof PrimitiveBase) {
      if (empty($field->getValue())) {
        $field = $defaultValue;
      } else {
        $field = $field->getString();
      }
    }

    return $field;
  }

  private function findPlugin($field) {
    foreach ($this->fieldResolvers as $resolver) {
      if ($resolver->can($field)) {
        return $resolver->handler();
      }
    }
  }

}
